"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = __importDefault(require("./index"));
const jszip_1 = __importDefault(require("jszip"));
exports.SUPPORTED_SAVE_VERSIONS = ["1.0.0"];
async function createSaveFrom(archive) {
    const info = archive.synthetic_info;
    function convertConversationToGDPRConversation(conversation) {
        return {
            dmConversation: {
                conversationId: conversation.id,
                messages: conversation.all
                    .map(message => ({
                    messageCreate: {
                        recipientId: message.recipientId,
                        createdAt: message.createdAt,
                        mediaUrls: message.mediaUrls,
                        text: message.text,
                        senderId: message.senderId,
                        id: message.id
                    }
                }))
            }
        };
    }
    const tweets = archive.all;
    for (const tweet of tweets) {
        delete tweet.created_at_d;
    }
    const tweet_zip = await new jszip_1.default().file("tweet.json", JSON.stringify(tweets)).generateAsync({
        type: "arraybuffer",
        compression: "DEFLATE",
        compressionOptions: {
            level: 6 // Not too much, if we want a good generation time
        }
    });
    const mutes = archive.extended_gdpr ? [...archive.extended_gdpr.mutes] : [];
    const blocks = archive.extended_gdpr ? [...archive.extended_gdpr.blocks] : [];
    let dms = null;
    if (archive.is_gdpr && archive.messages) {
        // Swallow copy all the dms, save them to a JSZip instance
        /*
          dm.json => [
            GDPRConversation,
            ...
          ]
        */
        dms = await new jszip_1.default()
            .file("dm.json", JSON.stringify(archive.messages.all.map(convertConversationToGDPRConversation)))
            .generateAsync({
            type: "arraybuffer",
            compression: "DEFLATE",
            compressionOptions: {
                level: 6 // Not too much, if we want a good generation time
            }
        });
    }
    return {
        tweets: tweet_zip,
        dms,
        info,
        mutes,
        blocks,
        screen_name_history: archive.extended_gdpr ? archive.extended_gdpr.screen_name_history : []
    };
}
exports.default = createSaveFrom;
async function createFromSave(save) {
    save = await save;
    if (!exports.SUPPORTED_SAVE_VERSIONS.includes(save.info.version)) {
        throw new Error("Save version is not supported.");
    }
    const archive = new index_1.default(null);
    archive.info.archive = save.info.info.archive;
    archive.info.user = save.info.info.user;
    const tweet_archive = await jszip_1.default.loadAsync(save.tweets);
    let current_load_object = JSON.parse(await tweet_archive.file("tweet.json").async("text"));
    // Tweets are extracted from a previous archive, they've been converted to classic format.
    archive.loadClassicArchivePart({ tweets: current_load_object });
    current_load_object = undefined;
    if (save.info.is_gdpr) {
        // Side effect of this method is to define archive to GDPR format
        await archive.loadArchivePart();
    }
    if (save.dms) {
        const dm_archive = await jszip_1.default.loadAsync(save.dms);
        current_load_object = JSON.parse(await dm_archive.file("dm.json").async("text"));
        await archive.loadArchivePart({
            dms: [current_load_object]
        });
    }
    if (archive.is_gdpr) {
        archive.extended_gdpr = {
            followers: new Set,
            followings: new Set,
            mutes: new Set(save.mutes),
            blocks: new Set(save.blocks),
            personalization: undefined,
            favorites: new Set,
            lists: {
                created: [],
                member_of: [],
                subscribed: []
            },
            screen_name_history: save.screen_name_history,
            protected_history: [],
            age_info: undefined,
            moments: []
        };
    }
    return archive;
}
exports.createFromSave = createFromSave;
